--[[****************************************************************
	GemWatch v0.91

	Author: Brandorf
	Based on the original addon by Evil Duck
	****************************************************************
	Description:
		Type /gw or /gemwatch to open and close the window
	****************************************************************]]
-- 0.91 Fixed bug where /gw toggle would not completely hide gem counters.  Fixed bug where initial scan after load would not always function properly.
-- 0.90 Updated for the 4.0 patch.  Gemwatch can now automatically determine if a character is a Jewelcrafter, and set the display of Dragoneyes appropriatley.  This can still be manually toggled with /gw Jeweler.  Integration with the new stat panels is coming in a future release.
-- 0.87 Really fixed the memory allocation bug.
-- 0.86 Some inventory scanning optimizations.
--      Added special layout considerations for CharSheetPlus and AhmUI, which only come into effect if those addons are loaded.
-- 0.85 Fixed a bug that could cause Gemwatch to crash WoW when used with addons such as GearScore.
-- 0.84 Updated Korean Localization 
-- 0.83 TOC Updated to 30300
-- 0.82 Russian(ruRU) translation data submitted by ChiffaFox.
-- 0.80 Added tracking of Jeweler's Gems (Dragon's Eye) this requires localization.
-- 0.76 French(frFR) partial translation data submitted by Ckeurk.
-- 0.72 Korean(koKR) Translation data submitted by Nukuge.
-- 0.71 German(deDE) translation data submitted by Sammler.
-- 0.70 Added localization support.
-- 0.65 Corrected positiong bug that can occur if a gear change happens in combat.
-- 0.61 Fixed bug with prismatic gem detection if in the first or second gem slots.
-- 0.60 Fixed detection of BS prismatic sockets.  Adjusted gem/socket icon layout.
-- 0.55 Added tracking of prismatic gems they are listed as their own category, not counted among the color sockets.
--      Now shows empty sockets by color (prismatic does not work, for some reason).
-- 0.45 Fixed positioning error
-- 0.40 Rewritten to take advantage of new 3.2 functionality (no need for EnchantID's)
-- 		can now track meta activation.
-- 0.30 Updated for WOTLK 3.2
-- 0.21 Some Meta's were counted incorrectly as blue.
-- 0.20 Updated all Gems up to patch 2.4.2
-- 0.19 Added options to position gem-icons
-- 0.18 Added option to show/hide gem-interface in character window
-- 0.17 Tweaked character-frame
-- 0.16 Fixed an erroneous gem, added more gems, changed character-window look
-- 0.14 Integrated in characterwindow
-- 0.10 First version

GEMWATCH_VERSIONTEXT = "GemWatch v0.86";
SLASH_GEMWATCH1 = "/gemwatch";
SLASH_GEMWATCH2 = "/gw";

local GemWatch_Red=0;
local GemWatch_Yellow=0;
local GemWatch_Blue=0;
local GemWatch_Meta=0;
local GemWatch_Purple=0;
local GemWatch_Green=0;
local GemWatch_Orange=0;
local GemWatch_Prismatic=0;
local GemWatch_DragonEye=0;
local GemWatch_Empty_Red=0;
local GemWatch_Empty_Yellow=0;
local GemWatch_Empty_Blue=0;
local GemWatch_Empty_Meta=0;
local GemWatch_Empty_Prismatic=0;
local GemWatch_InCombat = false;

local GemWatch_GearEmptyRed={};
local GemWatch_GearEmptyYellow={};
local GemWatch_GearEmptyBlue={};
local GemWatch_GearEmptyPrismatic={};
local GemWatch_GearEmptyMeta={};
local GemWatch_GearDragonEye={};

local L = GemWatch_Localization;

local gemFrame;

GemWatch_Settings = {
	PWHide = false,
	Position = 1,
	Debug = false,
	Override = false,
	Jeweler = false,
	JewelerChecked = false;
};

-- Set up for handling
function GemWatch_OnLoad()
	tinsert(UISpecialFrames,"GemWatch_Frame");
	GemWatch_Frame:RegisterEvent("ADDON_LOADED");
	--this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	GemWatch_Frame:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	GemWatch_Frame:RegisterEvent("PLAYER_REGEN_ENABLED");
	
	SlashCmdList["GEMWATCH"]=function(msg) GemWatch_Slasher(msg) end;
	
	GemWatch_Chat(L['GEMWATCH_LOADED']);	
end


-- There's slashing to be done
function GemWatch_Slasher(msg)
-- Validate input
	if (not msg) then msg=""; end
	if (strlen(msg)>0) then msg=strlower(msg); end

	if (msg=="toggle") then
		GemWatch_Settings.PWHide = not GemWatch_Settings.PWHide;
		GemWatch_SetPaperDoll();
		return;
	end
	
	if(msg=="debug") then
		GemWatch_Settings.Debug = not GemWatch_Settings.Debug;
		if(GemWatch_Settings.Debug ) then
			GemWatch_Chat(L['GEMWATCH_GUIDE_DEBUG_ON']);
		else
			GemWatch_Chat(L['GEMWATCH_GUIDE_DEBUG_OFF']);
		end
		return;
	end
	
	if(msg=="override") then
		GemWatch_Settings.Override= not GemWatch_Settings.Override;
		if(GemWatch_Settings.Override) then
			GemWatch_SetLocalizedGemColors();
			GemWatch_Chat(L['GEMWATCH_GUIDE_OVERRIDE_ON']);
		else
			GemWatch_Chat(L['GEMWATCH_GUIDE_OVERRIDE_OFF']);
		end		
		return;
	end
	
	if(msg=="jeweler") then
		GemWatch_Settings.Jeweler= not GemWatch_Settings.Jeweler;
		if(GemWatch_Settings.Jeweler) then
			GemWatch_Chat(L['GEMWATCH_GUIDE_JEWELER_ON']);
		else
			GemWatch_Chat(L['GEMWATCH_GUIDE_JEWELER_OFF']);
		end
		GemWatch_SetPosition(GemWatch_Settings.Position);
		return;
	end


	if (msg=="pos 1") then GemWatch_SetPosition(1); GemWatch_SetPaperDoll(); return; end
	if (msg=="pos 2") then GemWatch_SetPosition(2); GemWatch_SetPaperDoll(); return; end
	if (msg=="pos 3") then GemWatch_SetPosition(3); GemWatch_SetPaperDoll(); return; end
	if (msg=="pos 4") then GemWatch_SetPosition(4); GemWatch_SetPaperDoll(); return; end
	if (msg=="pos 5") then GemWatch_SetPosition(5); GemWatch_SetPaperDoll(); return; end
	if (msg=="pos 6") then GemWatch_SetPosition(6); GemWatch_SetPaperDoll(); return; end

	if (msg=="?") then
		GemWatch_Chat(GEMWATCH_VERSIONTEXT);
		GemWatch_Chat("Type \""..SLASH_GEMWATCH1.."\" or \""..SLASH_GEMWATCH2.."\" to toggle GemWatch edit-interface");
		GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_TOGGLE']);
		GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_POS']);
		GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_DEBUG']);
		GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_OVERRIDE']);
		GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_JEWELER']);
		return;
	end

	GemWatch_Chat(" --> "..SLASH_GEMWATCH2..L['GEMWATCH_GUIDE_HELP']);
end

function GemWatch_ResetResistFrame() -- obsolete
		MagicResFrame1:ClearAllPoints()
		MagicResFrame1:SetPoint("TOPLEFT", "CharacterResistanceFrame", "TOPLEFT", 0, 0)
		MagicResFrame2:ClearAllPoints()
		MagicResFrame2:SetPoint("TOP", "MagicResFrame1", "BOTTOM", 0, 0)
		MagicResFrame3:ClearAllPoints()
		MagicResFrame3:SetPoint("TOP", "MagicResFrame2", "BOTTOM", 0, 0)
		MagicResFrame4:ClearAllPoints()
		MagicResFrame4:SetPoint("TOP", "MagicResFrame3", "BOTTOM", 0, 0)
		MagicResFrame5:ClearAllPoints()
		MagicResFrame5:SetPoint("TOP", "MagicResFrame4", "BOTTOM", 0, 0)
end

function GemWatch_SetPosition(pos)
	if (not pos) then pos=1; end
	GemWatch_Settings.Position=pos;
	
		
	CharacterModelFrameGemsBlue:ClearAllPoints();
	CharacterModelFrameGemsYellow:ClearAllPoints();
	CharacterModelFrameGemsRed:ClearAllPoints();
	CharacterModelFrameGemsMeta:ClearAllPoints();
	CharacterModelFrameGemsPrism:ClearAllPoints();
	CharacterModelFrameGemsEmptyBlue:ClearAllPoints();
	CharacterModelFrameGemsEmptyYellow:ClearAllPoints();
	CharacterModelFrameGemsEmptyRed:ClearAllPoints();
	CharacterModelFrameGemsEmptyMeta:ClearAllPoints();
	CharacterModelFrameGemsEmptyPrism:ClearAllPoints();
	CharacterModelFrameGemsDragonEye:ClearAllPoints();
	
	if(GemWatch_Settings.Jeweler) and (not GemWatch_Settings.PWHide)then
		CharacterModelFrameGemsDragonEye:Show();
	else
		CharacterModelFrameGemsDragonEye:Hide();
	end
		
	if (pos==6) then
	
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler) then
			CharacterModelFrameGemsDragonEye:SetPoint("TOPRIGHT","CharacterModelFrame","TOPRIGHT",-3,0);
			CharacterModelFrameGemsBlue:SetPoint("RIGHT","CharacterModelFrameGemsDragonEye","LEFT",-4,0);
		else
			CharacterModelFrameGemsBlue:SetPoint("TOPRIGHT","CharacterModelFrame","TOPRIGHT",-3,-0);
		end
			
		CharacterModelFrameGemsYellow:SetPoint("RIGHT","CharacterModelFrameGemsBlue","LEFT",-4,0);
		CharacterModelFrameGemsRed:SetPoint("RIGHT","CharacterModelFrameGemsYellow","LEFT",-4,0);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		--IsVisible does not work, as this gets called even if the character sheet is closed.
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-4,0);
		else
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-4,-24);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,0);
		else
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,-24);
		end
		
		CharacterModelFrameGemsEmptyBlue:SetPoint("TOP","CharacterModelFrameGemsBlue","BOTTOM");
		CharacterModelFrameGemsEmptyYellow:SetPoint("TOP","CharacterModelFrameGemsYellow","BOTTOM");
		CharacterModelFrameGemsEmptyRed:SetPoint("TOP","CharacterModelFrameGemsRed","BOTTOM");
		CharacterModelFrameGemsEmptyMeta:SetPoint("TOP","CharacterModelFrameGemsMeta","BOTTOM");
		CharacterModelFrameGemsEmptyPrism:SetPoint("TOP","CharacterModelFrameGemsPrism","BOTTOM");
	
	elseif (pos==5) then
	
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler) then
			CharacterModelFrameGemsDragonEye:SetPoint("TOPRIGHT","CharacterModelFrame","TOPRIGHT", 0,-2);
			CharacterModelFrameGemsBlue:SetPoint("RIGHT","CharacterModelFrameGemsDragonEye","LEFT",-3,0);
		else
			CharacterModelFrameGemsBlue:SetPoint("TOPRIGHT","CharacterModelFrame","TOPRIGHT", 0,-2);
		end
			
		CharacterModelFrameGemsYellow:SetPoint("RIGHT","CharacterModelFrameGemsBlue","LEFT",-3,0);
		CharacterModelFrameGemsRed:SetPoint("RIGHT","CharacterModelFrameGemsYellow","LEFT",-3,0);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		--IsVisible does not work, as this gets called even if the character sheet is closed.
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-3,0);
		else
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-3,-24);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,0);
		else
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,-24);
		end
		
		CharacterModelFrameGemsEmptyBlue:SetPoint("BOTTOM","CharacterModelFrameGemsBlue","TOP");
		CharacterModelFrameGemsEmptyYellow:SetPoint("BOTTOM","CharacterModelFrameGemsYellow","TOP");
		CharacterModelFrameGemsEmptyRed:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP");
		CharacterModelFrameGemsEmptyMeta:SetPoint("BOTTOM","CharacterModelFrameGemsMeta","TOP");
		CharacterModelFrameGemsEmptyPrism:SetPoint("BOTTOM","CharacterModelFrameGemsPrism","TOP");
		
	elseif (pos==4) then
	
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler)then
			CharacterModelFrameGemsDragonEye:SetPoint("BOTTOMRIGHT","CharacterModelFrame","BOTTOMRIGHT",-3,26);
			CharacterModelFrameGemsBlue:SetPoint("RIGHT","CharacterModelFrameGemsDragonEye","LEFT",-3,0);
		else
			CharacterModelFrameGemsBlue:SetPoint("BOTTOMRIGHT","CharacterModelFrame","BOTTOMRIGHT",-3,26);
		end
			
		CharacterModelFrameGemsYellow:SetPoint("RIGHT","CharacterModelFrameGemsBlue","LEFT",-3,0);
		CharacterModelFrameGemsRed:SetPoint("RIGHT","CharacterModelFrameGemsYellow","LEFT",-3,0);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		--IsVisible does not work, as this gets called even if the character sheet is closed.
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-3,0);
		else
			CharacterModelFrameGemsPrism:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-3,-24);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,0);
		else
			CharacterModelFrameGemsMeta:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT",-30,-24);
		end
		
		CharacterModelFrameGemsEmptyBlue:SetPoint("BOTTOM","CharacterModelFrameGemsBlue","TOP");
		CharacterModelFrameGemsEmptyYellow:SetPoint("BOTTOM","CharacterModelFrameGemsYellow","TOP");
		CharacterModelFrameGemsEmptyRed:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP");
		CharacterModelFrameGemsEmptyMeta:SetPoint("BOTTOM","CharacterModelFrameGemsMeta","TOP");
		CharacterModelFrameGemsEmptyPrism:SetPoint("BOTTOM","CharacterModelFrameGemsPrism","TOP");

	elseif (pos==3) then
		
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler)then
			CharacterModelFrameGemsDragonEye:SetPoint("BOTTOMLEFT","CharacterModelFrame","BOTTOMLEFT",3,26);
			CharacterModelFrameGemsBlue:SetPoint("LEFT","CharacterModelFrameGemsDragonEye","RIGHT",3,0);
		else
			CharacterModelFrameGemsBlue:SetPoint("BOTTOMLEFT","CharacterModelFrame","BOTTOMLEFT",3,26);
		end
					
		CharacterModelFrameGemsYellow:SetPoint("LEFT","CharacterModelFrameGemsBlue","RIGHT",3,0);
		CharacterModelFrameGemsRed:SetPoint("LEFT","CharacterModelFrameGemsYellow","RIGHT",3,0);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("LEFT","CharacterModelFrameGemsRed","RIGHT",3,0);
		else
			CharacterModelFrameGemsPrism:SetPoint("LEFT","CharacterModelFrameGemsRed","RIGHT",3,-24);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("LEFT","CharacterModelFrameGemsRed","RIGHT",27,0);
		else
			CharacterModelFrameGemsMeta:SetPoint("LEFT","CharacterModelFrameGemsRed","RIGHT",27,-24);
		end
		
				
		CharacterModelFrameGemsEmptyBlue:SetPoint("BOTTOM","CharacterModelFrameGemsBlue","TOP");
		CharacterModelFrameGemsEmptyYellow:SetPoint("BOTTOM","CharacterModelFrameGemsYellow","TOP");
		CharacterModelFrameGemsEmptyRed:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP");
		CharacterModelFrameGemsEmptyMeta:SetPoint("BOTTOM","CharacterModelFrameGemsMeta","TOP");
		CharacterModelFrameGemsEmptyPrism:SetPoint("BOTTOM","CharacterModelFrameGemsPrism","TOP");
	
	elseif (pos==2) then
		
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler)then
			CharacterModelFrameGemsDragonEye:SetPoint("BOTTOMLEFT","CharacterModelFrame","BOTTOMLEFT",3,26);
			CharacterModelFrameGemsBlue:SetPoint("BOTTOM","CharacterModelFrameGemsDragonEye","TOP",0,3);
		else
			CharacterModelFrameGemsBlue:SetPoint("BOTTOMLEFT","CharacterModelFrame","BOTTOMLEFT",3,26);
		end
				
		CharacterModelFrameGemsYellow:SetPoint("BOTTOM","CharacterModelFrameGemsBlue","TOP",0,3);
		CharacterModelFrameGemsRed:SetPoint("BOTTOM","CharacterModelFrameGemsYellow","TOP",0,3);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP",0,3);
		else
			CharacterModelFrameGemsPrism:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP",-24,3);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP",0,27);
		else
			CharacterModelFrameGemsMeta:SetPoint("BOTTOM","CharacterModelFrameGemsRed","TOP",-24,27);
		end
		
		CharacterModelFrameGemsEmptyBlue:SetPoint("LEFT","CharacterModelFrameGemsBlue","RIGHT");
		CharacterModelFrameGemsEmptyYellow:SetPoint("LEFT","CharacterModelFrameGemsYellow","RIGHT");
		CharacterModelFrameGemsEmptyRed:SetPoint("LEFT","CharacterModelFrameGemsRed","RIGHT");
		CharacterModelFrameGemsEmptyMeta:SetPoint("LEFT","CharacterModelFrameGemsMeta","RIGHT");
		CharacterModelFrameGemsEmptyPrism:SetPoint("LEFT","CharacterModelFrameGemsPrism","RIGHT");
		
	else
		GemWatch_Settings.Position=1;
		-- hide Dragon's Eye counter if we don't want it.
		if(GemWatch_Settings.Jeweler)then
			CharacterModelFrameGemsDragonEye:SetPoint("RIGHT","CharacterModelFrame","TOPRIGHT",0,-12);
			CharacterModelFrameGemsBlue:SetPoint("TOP","CharacterModelFrameGemsDragonEye","BOTTOM",0,-4);
		else
			CharacterModelFrameGemsBlue:SetPoint("RIGHT","CharacterModelFrame","TOPRIGHT",0,-12);
		end
		CharacterModelFrameGemsYellow:SetPoint("TOP","CharacterModelFrameGemsBlue","BOTTOM",0, -4);
		CharacterModelFrameGemsRed:SetPoint("TOP","CharacterModelFrameGemsYellow","BOTTOM",0, -4);
		
		--if(CharacterModelFrameGemsPrism:IsVisible()) then
		if(GemWatch_Prismatic > 0) then
			CharacterModelFrameGemsPrism:SetPoint("TOP","CharacterModelFrameGemsRed","BOTTOM",0, -4);
		else
			CharacterModelFrameGemsPrism:SetPoint("TOP","CharacterModelFrameGemsRed","BOTTOM",0, -4);
		end
		
		--if(CharacterModelFrameGemsMeta:IsVisible()) then
		if(GemWatch_Meta > 0) then
			CharacterModelFrameGemsMeta:SetPoint("TOP","CharacterModelFrameGemsRed","BOTTOM",0, -32);
		else
			CharacterModelFrameGemsMeta:SetPoint("TOP","CharacterModelFrameGemsRed","BOTTOM",0, -4);
		end
								
		CharacterModelFrameGemsEmptyBlue:SetPoint("RIGHT","CharacterModelFrameGemsBlue","LEFT");
		CharacterModelFrameGemsEmptyYellow:SetPoint("RIGHT","CharacterModelFrameGemsYellow","LEFT");
		CharacterModelFrameGemsEmptyRed:SetPoint("RIGHT","CharacterModelFrameGemsRed","LEFT");
		CharacterModelFrameGemsEmptyMeta:SetPoint("RIGHT","CharacterModelFrameGemsMeta","LEFT");
		CharacterModelFrameGemsEmptyPrism:SetPoint("RIGHT","CharacterModelFrameGemsPrism","LEFT");
		
	end
end

function GemWatch_SetPaperDoll()
	if (GemWatch_Settings.PWHide==true) then
		CharacterModelFrameGemsBlue:Hide();
		CharacterModelFrameGemsYellow:Hide();
		CharacterModelFrameGemsRed:Hide();
		CharacterModelFrameGemsMeta:Hide();
		CharacterModelFrameGemsPrism:Hide();
		CharacterModelFrameGemsDragonEye:Hide();
		
		CharacterModelFrameGemsEmptyBlue:Hide();
		CharacterModelFrameGemsEmptyYellow:Hide();
		CharacterModelFrameGemsEmptyRed:Hide();
		CharacterModelFrameGemsEmptyMeta:Hide();
		CharacterModelFrameGemsEmptyPrism:Hide();
		
	else
		CharacterModelFrameGemsBlue:Show();
		CharacterModelFrameGemsYellow:Show();
		CharacterModelFrameGemsRed:Show();
		CharacterModelFrameGemsMeta:Show();
		CharacterModelFrameGemsPrism:Show();
		CharacterModelFrameGemsDragonEye:Show();
		
		CharacterModelFrameGemsEmptyBlue:Show();
		CharacterModelFrameGemsEmptyYellow:Show();
		CharacterModelFrameGemsEmptyRed:Show();
		CharacterModelFrameGemsEmptyMeta:Show();
		CharacterModelFrameGemsEmptyPrism:Show();
	end
end

-- Generic chat-pane stuff
function GemWatch_Chat(msg,r,g,b)
	if (DEFAULT_CHAT_FRAME) then
		if (not r and not g and not b) then r=1; g=1; b=1; end;
		if (not r) then r=0; end;
		if (not g) then g=0; end;
		if (not b) then b=0; end;
		DEFAULT_CHAT_FRAME:AddMessage("GemWatch: "..msg,r,g,b);
	end
end

function GemWatch_DebugChat(msg,r,g,b)
	if(not GemWatch_Settings.Debug) then
		return;
	end
	
	if (DEFAULT_CHAT_FRAME) then
		if (not r and not g and not b) then r=1; g=1; b=1; end;
		if (not r) then r=0; end;
		if (not g) then g=0; end;
		if (not b) then b=0; end;
		DEFAULT_CHAT_FRAME:AddMessage("GemWatch: "..msg,r,g,b);
	end
end

-- Duh
function GemWatch_ToggleMainFrame()
	if (GemWatch_Frame:IsVisible()) then
		GemWatch_Frame:Hide();
	else
		GemWatch_ScanInventory();
		GemWatch_Frame:Show();
	end
end

-- An event has been received
function GemWatch_OnEvent(self, event, ...)
	if (event=="ADDON_LOADED") then
		local addon = select(1, ...);
		if ( addon == "GemWatch" ) then
			--GemWatch_DebugChat("ADDON: "..arg1);
			GemWatch_DebugChat("Event: VARIABLES_LOADED");
			GemWatch_SetPaperDoll();
			GemWatch_ScanInventory();
			if(GemWatch_Settings.Override) then GemWatch_SetLocalizedGemColors(); end
			if (not GemWatch_Settings.JewelerChecked) then GemWatch_Settings.JewelerChecked = false; end
			if(GemWatch_Settings.JewelerChecked == false) then GemWatch_CheckIfJewelcrafter(); end
			GemWatch_SetPosition(GemWatch_Settings.Position);
		end
	end

	if (event=="UNIT_INVENTORY_CHANGED") and (arg1 == "player") then
		if(InCombatLockdown()) then
			GemWatch_InCombat = true;
		else
			GemWatch_ScanInventory();
			GemWatch_SetPosition(GemWatch_Settings.Position);
		end;
	end
	
	if (event=="PLAYER_EQUIPMENT_CHANGED")then
		if(InCombatLockdown()) then
			GemWatch_InCombat = true;
		else
			GemWatch_ScanInventory();
			GemWatch_SetPosition(GemWatch_Settings.Position);
		end;
	end
	
	if (event=="PLAYER_REGEN_ENABLED") then
		if(GemWatch_InCombat == true) then
			GemWatch_ScanInventory();
			GemWatch_SetPosition(GemWatch_Settings.Position);
			GemWatch_InCombat = false;
		end
	end
end

function GemWatch_SetLocalizedGemColors()
	local red, blue, yellow, purple, green, orange, meta, simple, prismatic =  GetAuctionItemSubClasses(10); --List of all gem subclasses, red, ble, meta, et.al

	-- Override what's in the localized string table
	L['GEMWATCH_RED_GEM'] = red;
	L['GEMWATCH_BLUE_GEM'] = blue;
	L['GEMWATCH_YELLOW_GEM'] = yellow;
	L['GEMWATCH_PURPLE_GEM'] = purple;
	L['GEMWATCH_GREEN_GEM'] = green;
	L['GEMWATCH_ORANGE_GEM'] = orange;
	L['GEMWATCH_META_GEM'] = meta;
	L['GEMWATCH_SIMPLE_GEM'] = simple;
	L['GEMWATCH_PRISMATIC_GEM'] = prismatic;
		
end

function GemWatch_MetaButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsMeta,"ANCHOR_RIGHT");
	local _, gemLink = GetItemInfo(GemWatch_Metalink); 
	GameTooltip:SetHyperlink(gemLink);
	GameTooltip:Show();
end

function GemWatch_DragonEyeButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsDragonEye,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_CONTRIBUTION'].." "..L['GEMWATCH_DRAGONEYE_GEM'].." "..L['GEMWATCH_GEMS']..".");
	if(GemWatch_DragonEye > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_DRAGONEYE_SOCKET'],1,1,1);
		for i,v in ipairs(GemWatch_GearDragonEye) do GameTooltip:AddLine(v) end
	end;
	
	GameTooltip:Show();
end

function GemWatch_RedButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsRed,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_CONTRIBUTION'].." "..L['GEMWATCH_RED_GEM'].." "..L['GEMWATCH_GEMS']..".");
	if((GemWatch_Red - GemWatch_Orange - GemWatch_Purple) > 0)then GameTooltip:AddLine(L['GEMWATCH_RED_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..(GemWatch_Red - GemWatch_Orange - GemWatch_Purple)..")",1,0,0); end;
	if(GemWatch_Orange > 0)then GameTooltip:AddLine(L['GEMWATCH_ORANGE_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Orange..")",1,0.5,0); end;
	if(GemWatch_Purple > 0)then GameTooltip:AddLine(L['GEMWATCH_PURPLE_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Purple..")",0.5,0,1); end;
	GameTooltip:Show();
end

function GemWatch_YellowButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsYellow,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_CONTRIBUTION'].." "..L['GEMWATCH_YELLOW_GEM'].." "..L['GEMWATCH_GEMS']..".");
	if((GemWatch_Yellow - GemWatch_Orange - GemWatch_Green) > 0)then GameTooltip:AddLine(L['GEMWATCH_YELLOW_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..(GemWatch_Yellow - GemWatch_Orange - GemWatch_Green)..")",1,1,0); end;
	if(GemWatch_Orange > 0)then GameTooltip:AddLine(L['GEMWATCH_ORANGE_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Orange..")",1,0.5,0); end;
	if(GemWatch_Green > 0)then GameTooltip:AddLine(L['GEMWATCH_GREEN_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Green..")",0,1,0); end;
	GameTooltip:Show();
end

function GemWatch_BlueButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsBlue,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_CONTRIBUTION'].." "..L['GEMWATCH_BLUE_GEM'].." "..L['GEMWATCH_GEMS']..".");
	if((GemWatch_Blue - GemWatch_Purple - GemWatch_Green) > 0)then GameTooltip:AddLine(L['GEMWATCH_BLUE_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..(GemWatch_Blue - GemWatch_Purple - GemWatch_Green)..")",0.2,0.2,1); end;
	if(GemWatch_Purple > 0)then GameTooltip:AddLine(L['GEMWATCH_PURPLE_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Purple..")",0.5,0,1); end;
	if(GemWatch_Green > 0)then GameTooltip:AddLine(L['GEMWATCH_GREEN_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Green..")",0,1,0); end;
	GameTooltip:Show();
end

function GemWatch_PrismButton_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsPrism,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_CONTRIBUTION'].." "..L['GEMWATCH_PRISMATIC_GEM'].." "..L['GEMWATCH_GEMS']..".");
	if(GemWatch_Prismatic > 0)then GameTooltip:AddLine(L['GEMWATCH_PRISMATIC_GEM'].." "..L['GEMWATCH_GEMS']..":".."("..GemWatch_Prismatic..")",1,1,1); end;
	GameTooltip:Show();
end

function GemWatch_EmptyRed_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsEmptyRed,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_EMPTY_RED_SOCKETS_LABEL']);
	if(GemWatch_Empty_Red > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_EMPTY_RED_SOCKETS'],1,1,1);
		for i,v in ipairs(GemWatch_GearEmptyRed) do GameTooltip:AddLine(v) end
	end;
	GameTooltip:Show();
end

function GemWatch_EmptyYellow_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsEmptyYellow,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_EMPTY_YELLOW_SOCKETS_LABEL']);
	if(GemWatch_Empty_Yellow > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_EMPTY_YELLOW_SOCKETS'],1,1,1);
		for i,v in ipairs(GemWatch_GearEmptyYellow) do GameTooltip:AddLine(v) end
	end;
	GameTooltip:Show();
end

function GemWatch_EmptyBlue_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsEmptyBlue,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_EMPTY_BLUE_SOCKETS_LABEL']);
	if(GemWatch_Empty_Blue > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_EMPTY_BLUE_SOCKETS'],1,1,1);
		for i,v in ipairs(GemWatch_GearEmptyBlue) do GameTooltip:AddLine(v) end
	end;
	GameTooltip:Show();
end

function GemWatch_EmptyMeta_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsEmptyMeta,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_EMPTY_META_SOCKET_LABEL']);
	if(GemWatch_Empty_Meta > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_EMPTY_META_SOCKET'],1,1,1);
		for i,v in ipairs(GemWatch_GearEmptyMeta) do GameTooltip:AddLine(v) end
	end;
	GameTooltip:Show();
end

function GemWatch_EmptyPrism_OnEnter()
	GameTooltip:SetOwner(CharacterModelFrameGemsEmptyPrism,"ANCHOR_RIGHT");
	GameTooltip:SetText(L['GEMWATCH_EMPTY_PRISMATIC_SOCKETS_LABEL']);
	if(GemWatch_Empty_Prismatic > 0) then 
		GameTooltip:AddLine(L['GEMWATCH_EMPTY_PRISMATIC_SOCKETS'],1,1,1);
		for i,v in ipairs(GemWatch_GearEmptyPrismatic) do GameTooltip:AddLine(v) end
	end;
	GameTooltip:Show();
end

function GemWatch_ScanInventory()
	GemWatch_DebugChat(L['GEMWATCH_DEBUG_INV_SCAN']);
	
	GemWatch_Red=0;
	GemWatch_Yellow=0;
	GemWatch_Blue=0;
	GemWatch_Meta=0;
	GemWatch_Purple=0;
	GemWatch_Green=0;
	GemWatch_Orange=0;
	GemWatch_Prismatic=0;
	GemWatch_DragonEye=0;
	GemWatch_Empty_Red=0;
	GemWatch_Empty_Yellow=0;
	GemWatch_Empty_Blue=0;
	GemWatch_Empty_Meta=0;
	GemWatch_Empty_Prismatic=0;
	GemWatch_Metalink=nil;
	wipe(GemWatch_GearEmptyRed);
	wipe(GemWatch_GearEmptyYellow);
	wipe(GemWatch_GearEmptyBlue);
	wipe(GemWatch_GearEmptyPrismatic);
	wipe(GemWatch_GearEmptyMeta);
	wipe(GemWatch_GearDragonEye);
	
	--local slotcount=1;
	
	local slot = 1; while(slot < 19) do
	
		-- Exclusions for slots we don't care about, or slots that can't possibly have gems.
		-- 0 is ammo, 19 is tabard, and 20 is a bag.
		
		if (slot ~= 4) then -- skip the shirt
			local gem1, gem2, gem3 = GetInventoryItemGems(slot);
		
			if(gem1) then 
				local name, _, _, _, _, class, subclass, _, _, _, _ = GetItemInfo(gem1);
			
				if(name) then  --check to see if this is a jewler gem
					if(string.find(name, L['GEMWATCH_DRAGONEYE_GEM'])) then 
						GemWatch_DragonEye = GemWatch_DragonEye + 1; 
						GemWatch_DebugChat(L['GEMWATCH_DEBUG_FOUND_DRAGONEYE']..GetInventoryItemLink("player",slot));  
						table.insert(GemWatch_GearDragonEye, GetInventoryItemLink("player",slot));
					end;
				end
			
				if(subclass == L['GEMWATCH_RED_GEM']) then GemWatch_Red = GemWatch_Red + 1;
				elseif(subclass == L['GEMWATCH_ORANGE_GEM']) then GemWatch_Orange = GemWatch_Orange + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_YELLOW_GEM']) then GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_GREEN_GEM']) then GemWatch_Green = GemWatch_Green + 1; GemWatch_Yellow = GemWatch_Yellow + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_BLUE_GEM']) then GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_PURPLE_GEM']) then GemWatch_Purple = GemWatch_Purple + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_META_GEM']) then GemWatch_Meta = GemWatch_Meta + 1; GemWatch_Metalink = gem1;
				elseif(subclass == L['GEMWATCH_PRISMATIC_GEM']) then GemWatch_Prismatic = GemWatch_Prismatic + 1;
				end;
			end;
		
			if(gem2) then 
				local name, _, _, _, _, class, subclass, _, _, _, _ = GetItemInfo(gem2);
				
				if(name) then  --check to see if this is a jewler gem
					if(string.find(name, L['GEMWATCH_DRAGONEYE_GEM'])) then 
						GemWatch_DragonEye = GemWatch_DragonEye + 1; 
						GemWatch_DebugChat(L['GEMWATCH_DEBUG_FOUND_DRAGONEYE']..GetInventoryItemLink("player",slot));
						table.insert(GemWatch_GearDragonEye, GetInventoryItemLink("player",slot));
					end;
				end
				
				if(subclass == L['GEMWATCH_RED_GEM']) then GemWatch_Red = GemWatch_Red + 1;
				elseif(subclass == L['GEMWATCH_ORANGE_GEM']) then GemWatch_Orange = GemWatch_Orange + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_YELLOW_GEM']) then GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_GREEN_GEM']) then GemWatch_Green = GemWatch_Green + 1; GemWatch_Yellow = GemWatch_Yellow + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_BLUE_GEM']) then GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_PURPLE_GEM']) then GemWatch_Purple = GemWatch_Purple + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_META_GEM']) then GemWatch_Meta = GemWatch_Meta + 1; GemWatch_Metalink = gem2;
				elseif(subclass == L['GEMWATCH_PRISMATIC_GEM']) then GemWatch_Prismatic = GemWatch_Prismatic + 1;
				end;
			end;
			
			if(gem3) then 
				local name, _, _, _, _, class, subclass, _, _, _, _ = GetItemInfo(gem3);
				
				if(name) then  --check to see if this is a jewler gem
					if(string.find(name, L['GEMWATCH_DRAGONEYE_GEM'])) then 
						GemWatch_DragonEye = GemWatch_DragonEye + 1; 
						GemWatch_DebugChat(L['GEMWATCH_DEBUG_FOUND_DRAGONEYE']..GetInventoryItemLink("player",slot)); 
						table.insert(GemWatch_GearDragonEye, GetInventoryItemLink("player",slot));
					end;
				end
				
				if(subclass == L['GEMWATCH_RED_GEM']) then GemWatch_Red = GemWatch_Red + 1;
				elseif(subclass == L['GEMWATCH_ORANGE_GEM']) then GemWatch_Orange = GemWatch_Orange + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_YELLOW_GEM']) then GemWatch_Yellow = GemWatch_Yellow + 1;
				elseif(subclass == L['GEMWATCH_GREEN_GEM']) then GemWatch_Green = GemWatch_Green + 1; GemWatch_Yellow = GemWatch_Yellow + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_BLUE_GEM']) then GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_PURPLE_GEM']) then GemWatch_Purple = GemWatch_Purple + 1; GemWatch_Red = GemWatch_Red + 1; GemWatch_Blue = GemWatch_Blue + 1;
				elseif(subclass == L['GEMWATCH_META_GEM']) then GemWatch_Meta = GemWatch_Meta + 1; GemWatch_Metalink = gem3;
				elseif(subclass == L['GEMWATCH_PRISMATIC_GEM']) then GemWatch_Prismatic = GemWatch_Prismatic + 1;
				end;
			end;
		end;
		GemWatch_CheckEmptySocketBySlot(slot);
		slot=slot+1;
	end;
	
	CharacterModelFrameGemsRedText:SetText(GemWatch_Red);
	CharacterModelFrameGemsYellowText:SetText(GemWatch_Yellow);
	CharacterModelFrameGemsBlueText:SetText(GemWatch_Blue);
	CharacterModelFrameGemsDragonEyeText:SetText(GemWatch_DragonEye);
	
	-- Prismatic gems display check
	if(GemWatch_Prismatic > 0) then
		CharacterModelFrameGemsPrismText:SetText(GemWatch_Prismatic)
		if (not GemWatch_Settings.PWHide) then 
			CharacterModelFrameGemsPrism:Show();
		end
	else
		CharacterModelFrameGemsPrism:Hide();
	end
	
	--Meta gems display check
	if (GemWatch_Meta > 0) then 
		if (not GemWatch_Settings.PWHide) then
			CharacterModelFrameGemsMeta:Show();
		end
		local activeMeta = GemWatch_Checkmeta();
		if (activeMeta == true) then
			CharacterModelFrameGemsMetaBlendTexture:SetTexture(0.0,0.0,0.0,0.3)
			CharacterModelFrameGemsMetaBlendTexture:SetBlendMode("BLEND");
			CharacterModelFrameGemsMetaText:SetText("")
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_META_ACTIVE'])
		else
			CharacterModelFrameGemsMetaBlendTexture:SetTexture(0.6,0.0,0.0,0.5)
			CharacterModelFrameGemsMetaBlendTexture:SetBlendMode("MOD");
			CharacterModelFrameGemsMetaText:SetText("")
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_META_INACTIVE'])
		end
	else
		CharacterModelFrameGemsMeta:Hide(); 
		CharacterModelFrameGemsMeta:SetBackdropColor(1.0,0.0,0.0);
	end
	
	-- Empty Sockets display checks
	if(GemWatch_Empty_Red > 0) then
		CharacterModelFrameGemsEmptyRedText:SetText(GemWatch_Empty_Red)
		CharacterModelFrameGemsEmptyRed:Show();
	else
		CharacterModelFrameGemsEmptyRed:Hide();
	end
	
	if(GemWatch_Empty_Yellow > 0) then
		CharacterModelFrameGemsEmptyYellowText:SetText(GemWatch_Empty_Yellow)
		CharacterModelFrameGemsEmptyYellow:Show();
	else
		CharacterModelFrameGemsEmptyYellow:Hide();
	end
	
	if(GemWatch_Empty_Blue > 0) then
		CharacterModelFrameGemsEmptyBlueText:SetText(GemWatch_Empty_Blue)
		CharacterModelFrameGemsEmptyBlue:Show();
	else
		CharacterModelFrameGemsEmptyBlue:Hide();
	end
	
	if(GemWatch_Empty_Meta > 0) then
		CharacterModelFrameGemsEmptyMetaText:SetText(GemWatch_Empty_Meta)
		CharacterModelFrameGemsEmptyMeta:Show();
	else
		CharacterModelFrameGemsEmptyMeta:Hide();
	end
	
	if(GemWatch_Empty_Prismatic > 0) then
		CharacterModelFrameGemsEmptyPrismText:SetText(GemWatch_Empty_Prismatic)
		CharacterModelFrameGemsEmptyPrism:Show();
	else
		CharacterModelFrameGemsEmptyPrism:Hide();
	end
	
end;

function GemWatch_Checkmeta()
	local f = CreateFrame('GameTooltip', 'MetaTooltip', UIParent, 'GameTooltipTemplate');
	local _, gemLink = GetItemInfo(GemWatch_Metalink); 
	GemWatch_DebugChat(L['GEMWATCH_DEBUG_META_SCAN'].. gemLink);
	f:SetOwner(UIParent, 'ANCHOR_NONE');
	f:SetHyperlink(gemLink);
	local metaLine = f:NumLines() - 2;
	local toolText = getglobal("MetaTooltipTextLeft" .. metaLine);
	local reqText = toolText:GetText();
	f:Hide();
	if( string.match(reqText, "\124cff808080") ) then
		return false;
	else
		return true;
	end
end;

function GemWatch_CheckEmptySocket(itemLink)
	if(itemLink == nil) then
		return
	end
	
	--GemWatch_DebugChat("Checking "..itemLink.." for empty sockets.");
	
	--local f = CreateFrame('GameTooltip', 'SocketTooltip', UIParent, 'GameTooltipTemplate');
	if (gemFrame == nil) then
		gemFrame = CreateFrame('GameTooltip', 'SocketTooltip', UIParent, 'GameTooltipTemplate');
	end
	
	-- clear the textures first
	for textureCount = 1, 10 do
		if _G["SocketTooltipTexture"..textureCount] then
			_G["SocketTooltipTexture"..textureCount]:SetTexture("");
		end
	end 
		
	gemFrame:SetOwner(UIParent, 'ANCHOR_NONE');
	gemFrame:ClearLines();
	gemFrame:SetHyperlink(itemLink);
		
	for textureCount = 1, 10 do
		local temp = _G["SocketTooltipTexture"..textureCount]:GetTexture();
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Meta" then 
			GemWatch_Empty_Meta = GemWatch_Empty_Meta + 1
			table.insert(GemWatch_GearEmptyMeta, itemLink);
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_META']..itemLink);
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Red" then 
			GemWatch_Empty_Red = GemWatch_Empty_Red + 1
			table.insert(GemWatch_GearEmptyRed, itemLink);
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_RED']..itemLink);
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Yellow" then 
			GemWatch_Empty_Yellow = GemWatch_Empty_Yellow + 1
			table.insert(GemWatch_GearEmptyYellow, itemLink);
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_YELLOW']..itemLink);			
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Blue" then 
			GemWatch_Empty_Blue = GemWatch_Empty_Blue + 1
			table.insert(GemWatch_GearEmptyBlue, itemLink);
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_BLUE']..itemLink);
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket" then 
			GemWatch_Empty_Prismatic = GemWatch_Empty_Prismatic + 1
			table.insert(GemWatch_GearEmptyPrismatic, itemLink);
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_PRISM']..itemLink);
		end 
	end
	gemFrame:Hide();
end;

function GemWatch_CheckEmptySocketBySlot(slot)
		
	--GemWatch_DebugChat("Checking "..GetInventoryItemLink("player",slot).." for empty sockets.");
	
	--local f = CreateFrame('GameTooltip', 'SocketTooltip', UIParent, 'GameTooltipTemplate');
	
	-- init my frame if it doesn't exist yet, otherwise reuse it.
	if (gemFrame == nil) then
		gemFrame = CreateFrame('GameTooltip', 'SocketTooltip', UIParent, 'GameTooltipTemplate');
	end
	
	-- clear the textures first
	for textureCount = 1, 10 do
		if _G["SocketTooltipTexture"..textureCount] then
			_G["SocketTooltipTexture"..textureCount]:SetTexture("");
		end
	end 
		
	gemFrame:SetOwner(UIParent, 'ANCHOR_NONE');
	gemFrame:ClearLines();
	gemFrame:SetInventoryItem("player",slot,false);
		
	for textureCount = 1, 10 do
		local temp = _G["SocketTooltipTexture"..textureCount]:GetTexture();
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Meta" then 
			GemWatch_Empty_Meta = GemWatch_Empty_Meta + 1
			table.insert(GemWatch_GearEmptyMeta, GetInventoryItemLink("player",slot));
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_META']..GetInventoryItemLink("player",slot));
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Red" then 
			GemWatch_Empty_Red = GemWatch_Empty_Red + 1
			table.insert(GemWatch_GearEmptyRed, GetInventoryItemLink("player",slot));
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_RED']..GetInventoryItemLink("player",slot));
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Yellow" then 
			GemWatch_Empty_Yellow = GemWatch_Empty_Yellow + 1
			table.insert(GemWatch_GearEmptyYellow, GetInventoryItemLink("player",slot));
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_YELLOW']..GetInventoryItemLink("player",slot));			
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket-Blue" then 
			GemWatch_Empty_Blue = GemWatch_Empty_Blue + 1
			table.insert(GemWatch_GearEmptyBlue, GetInventoryItemLink("player",slot));
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_BLUE']..GetInventoryItemLink("player",slot));
		end
		if temp and temp == "Interface\\ItemSocketingFrame\\UI-EmptySocket" then 
			GemWatch_Empty_Prismatic = GemWatch_Empty_Prismatic + 1
			table.insert(GemWatch_GearEmptyPrismatic, GetInventoryItemLink("player",slot));
			GemWatch_DebugChat(L['GEMWATCH_DEBUG_EMPTY_PRISM']..GetInventoryItemLink("player",slot));
		end 
	end
	gemFrame:Hide();
end;

-- Layout fix for CharSheetPlus
--[[local OldCharSheetToggle = CharSheetToggle;
function CharSheetToggle(...)
	OldCharSheetToggle(...);
	--GemWatch_Chat("Hooking call to CharSheetToggle()");
	GemWatch_SetPosition(GemWatch_Settings.Position);
end;--]]

function GemWatch_CheckIfJewelcrafter()
	GemWatch_DebugChat("Checking Professions...");
	local prof1, prof2 = GetProfessions();
	GemWatch_Settings.JewelerChecked = true;
	if (prof1) then
		local name = GetProfessionInfo(prof1);
		GemWatch_DebugChat("Profession #1 is "..name);
		if (name == "Jewelcrafting") then
			GemWatch_Settings.Jeweler = true;
			return
		end
	end
	
	if (prof2) then
		local name = GetProfessionInfo(prof2);
		GemWatch_DebugChat("Profession #2 is "..name);
		if (name == "Jewelcrafting") then
			GemWatch_Settings.Jeweler = true;
			return
		end
	end
	
	GemWatch_DebugChat(L['GEMWATCH_DEBUG_PROFESSION_NOTJC']);
	GemWatch_Settings.Jeweler = false;
	
end

